@@ -252,6 +252,52 @@ int parse_htpl_ldap(stack, untag)
     return 0;
 }
 
+int parse_htpl_rem___fwd(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    printcode("&begintransaction;\n");
+    nesting = 0;
+    RETURN(1)
+}
+
+int parse_htpl_rem___rev(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    printcode("&endtransaction;\n");
+    nesting = 0;
+    RETURN(1)
+}
+
+int parse_htpl_rem(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    if (!untag) return parse_htpl_rem___fwd(stack, untag);
+        else return parse_htpl_rem___rev(stack, untag);
+}
+
 int parse_htpl_include(stack, untag)
     int untag;
     STR stack; {
@@ -327,6 +373,64 @@ int parse_htpl_break(stack, untag)
     RETURN(1)
 }
 
+int parse_htpl_text_fixed(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    if (numtokens < 3) RETURN(croak("TEXT FIXED called with %d arguments, minimum needed is 3", numtokens))
+    code = 1;
+    buff = (STR)mysprintf("TEXT PREFIXED");
+    nest++;
+    code = parse_htpl(buff, untag);
+    nest--;
+    if (!code) {
+        croak("Unification of '%s' failed", buff);
+        free(buff);
+        RETURN(0)
+    }
+    free(buff);
+
+    printfcode("$%s = &HTML::HTPL::Fixed'openfixed(\"%s\", qw(%s));\n", gettoken(1), gettoken(2), gettokenlist(3, " ", ""));
+    nesting = 0;
+    RETURN(1)
+}
+
+int parse_htpl_text_records(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    if (numtokens < 3) RETURN(croak("TEXT RECORDS called with %d arguments, minimum needed is 3", numtokens))
+    code = 1;
+    buff = (STR)mysprintf("TEXT PREFIXED");
+    nest++;
+    code = parse_htpl(buff, untag);
+    nest--;
+    if (!code) {
+        croak("Unification of '%s' failed", buff);
+        free(buff);
+        RETURN(0)
+    }
+    free(buff);
+
+    printfcode("$%s = &HTML::HTPL::Fixed'openfixed(\"%s\", \\\"IBM\", qw(%s));\n", gettoken(1), gettoken(2), gettokenlist(3, " ", ""));
+    nesting = 0;
+    RETURN(1)
+}
+
 int parse_htpl_text_precsv(stack, untag)
     int untag;
     STR stack; {
@@ -347,6 +451,26 @@ int parse_htpl_text_precsv(stack, untag)
     RETURN(1)
 }
 
+int parse_htpl_text_prefixed(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    if (!nest) RETURN(0)
+    if (!done) {
+        done = 1;
+        printcode("use HTML::HTPL::Fixed;\n");
+    }
+    nesting = 0;
+    RETURN(1)
+}
+
 int parse_htpl_text_flat(stack, untag)
     int untag;
     STR stack; {
@@ -368,6 +492,65 @@ int parse_htpl_text_flat(stack, untag)
     RETURN(1)
 }
 
+int parse_htpl_text_template___fwd(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    if (numtokens < 1) RETURN(croak("TEXT TEMPLATE called with %d arguments, minimum needed is 1", numtokens))
+    if (numtokens > 1) RETURN(croak("TEXT TEMPLATE called with %d arguments, maximum needed is 1", numtokens))
+
+    printcode("{\n");
+    printfcode("my $__htpl_params = \\%%%s;\n", gettoken(1));
+    printcode("	&begintransaction;\n");
+    nesting = 0;
+    RETURN(1)
+}
+
+int parse_htpl_text_template___rev(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    printcode("{ my $text = &endtransaction;\n");
+    printcode("		 my $temp = new Template({ 'INCLUDE_PATH' => $ORIG_DIR,\n");
+    printcode("			'INTERPOLATE' => 1, 'EVAL_PERL' => 1});\n");
+    printcode("		$temp->process(\\$text, $__htpl_params); }\n");
+    printcode("}\n");
+    nesting = 0;
+    RETURN(1)
+}
+
+int parse_htpl_text_template(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    if (!done) {
+        done = 1;
+        printcode("use Template;\n");
+    }
+    if (!untag) return parse_htpl_text_template___fwd(stack, untag);
+        else return parse_htpl_text_template___rev(stack, untag);
+}
+
 int parse_htpl_text_read(stack, untag)
     int untag;
     STR stack; {
@@ -457,8 +640,12 @@ int parse_htpl_text(stack, untag)
     static int nesting = 0;
 
     eat(&stack, token);
+    if (!strcasecmp(token, "FIXED")) return parse_htpl_text_fixed(stack, untag);
+    if (!strcasecmp(token, "RECORDS")) return parse_htpl_text_records(stack, untag);
     if (!strcasecmp(token, "PRECSV")) return parse_htpl_text_precsv(stack, untag);
+    if (!strcasecmp(token, "PREFIXED")) return parse_htpl_text_prefixed(stack, untag);
     if (!strcasecmp(token, "FLAT")) return parse_htpl_text_flat(stack, untag);
+    if (!strcasecmp(token, "TEMPLATE")) return parse_htpl_text_template(stack, untag);
     if (!strcasecmp(token, "READ")) return parse_htpl_text_read(stack, untag);
     if (!strcasecmp(token, "CSV")) return parse_htpl_text_csv(stack, untag);
     if (!strcasecmp(token, "CUBE")) return parse_htpl_text_cube(stack, untag);
@@ -637,6 +824,25 @@ int parse_htpl_time(stack, untag)
     return 0;
 }
 
+int parse_htpl_sql_batch(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    if (numtokens < 2) RETURN(croak("SQL BATCH called with %d arguments, minimum needed is 2", numtokens))
+    if (numtokens > 2) RETURN(croak("SQL BATCH called with %d arguments, maximum needed is 2", numtokens))
+
+    printfcode("$HTML::HTPL::htpl_db_obj->batch_insert('%s', $%s);\n", gettoken(1), gettoken(2));
+    nesting = 0;
+    RETURN(1)
+}
+
 int parse_htpl_sql_update(stack, untag)
     int untag;
     STR stack; {
@@ -825,7 +1031,7 @@ int parse_htpl_sql_declare(stack, untag)
     static int nesting = 0;
 
     makepersist(stack);
-    printfcode("$HTML::HTPL::Sys::query_pool('%s'} ||= $HTML::HTPL::htpl_db_obj->prepare(\"%2*\");\n", gettoken(1));
+    printfcode("$HTML::HTPL::Sys::query_pool{'%s'} ||= $HTML::HTPL::htpl_db_obj->prepare(\"%s\");\n", gettoken(1), gettokenlist(2, " ", ""));
     nesting = 0;
     RETURN(1)
 }
@@ -1025,34 +1231,6 @@ int parse_htpl_sql_search(stack, untag)
     RETURN(1)
 }
 
-int parse_htpl_sql_postgres(stack, untag)
-    int untag;
-    STR stack; {
-
-    TOKEN token;
-    static done = 0;
-    STR buff;
-    int code;
-    static int nesting = 0;
-
-    makepersist(stack);
-    nesting++;
-    if (nesting > 1) RETURN(croak("Infinite loop in SQL POSTGRES"))
-    buff = (STR)mysprintf("SQL POSTGRESQL %s", gettokenlist(1, " ", ""));
-    nest++;
-    code = parse_htpl(buff, untag);
-    nest--;
-    if (!code) {
-        croak("Unification of '%s' failed", buff);
-        free(buff);
-        RETURN(0)
-    }
-    free(buff);
-
-    nesting = 0;
-    RETURN(1)
-}
-
 int parse_htpl_sql_project(stack, untag)
     int untag;
     STR stack; {
@@ -1064,6 +1242,7 @@ int parse_htpl_sql_project(stack, untag)
     static int nesting = 0;
 
     makepersist(stack);
+    if (numtokens < 2) RETURN(croak("SQL PROJECT called with %d arguments, minimum needed is 2", numtokens))
     printcode("{ my $imm;\n");
     code = 1;
     buff = (STR)mysprintf("SQL CURSOR imm %s", gettokenlist(2, " ", ""));
@@ -1094,6 +1273,151 @@ int parse_htpl_sql_project(stack, untag)
     RETURN(1)
 }
 
+int parse_htpl_sql_postgres(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    nesting++;
+    if (nesting > 1) RETURN(croak("Infinite loop in SQL POSTGRES"))
+    buff = (STR)mysprintf("SQL POSTGRESQL %s", gettokenlist(1, " ", ""));
+    nest++;
+    code = parse_htpl(buff, untag);
+    nest--;
+    if (!code) {
+        croak("Unification of '%s' failed", buff);
+        free(buff);
+        RETURN(0)
+    }
+    free(buff);
+
+    nesting = 0;
+    RETURN(1)
+}
+
+int parse_htpl_sql_scope_connect(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    if (numtokens < 1) RETURN(croak("SQL SCOPE CONNECT called with %d arguments, minimum needed is 1", numtokens))
+    if (numtokens > 3) RETURN(croak("SQL SCOPE CONNECT called with %d arguments, maximum needed is 3", numtokens))
+
+    setvar("dbobj", (STR)mysprintf("$__htpl_db_%s_%d", getblockid(""), random()));
+    if (!exportvar("dbobj", "")) RETURN(croak("Scope  not found in stack"));
+    printfcode("my %s = \n", getvar("dbobj"));
+    printfcode("   HTML::HTPL::Db->new(\"%s\", \"%s\", \"%s\");\n", gettoken(1), gettoken(2), gettoken(3));
+    printcode("		\n");
+    nesting = 0;
+    RETURN(1)
+}
+
+int parse_htpl_sql_scope_exec(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    if (numtokens < 1) RETURN(croak("SQL SCOPE EXEC called with %d arguments, minimum needed is 1", numtokens))
+    code = 1;
+    buff = (STR)mysprintf("SQL SCOPE RETRIEVE");
+    nest++;
+    code = parse_htpl(buff, untag);
+    nest--;
+    if (!code) {
+        croak("Unification of '%s' failed", buff);
+        free(buff);
+        RETURN(0)
+    }
+    free(buff);
+
+    printfcode("%s->execsql(&HTML::HTPL::Db'parse_sql(\"%s\"));\n", getvar("dbobj"), gettokenlist(1, " ", ""));
+    nesting = 0;
+    RETURN(1)
+}
+
+int parse_htpl_sql_scope_cursor(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    if (numtokens < 2) RETURN(croak("SQL SCOPE CURSOR called with %d arguments, minimum needed is 2", numtokens))
+    code = 1;
+    buff = (STR)mysprintf("SQL SCOPE RETRIEVE");
+    nest++;
+    code = parse_htpl(buff, untag);
+    nest--;
+    if (!code) {
+        croak("Unification of '%s' failed", buff);
+        free(buff);
+        RETURN(0)
+    }
+    free(buff);
+
+    printfcode("$%s = %s->cursor(&HTML::HTPL::Db'parse_sql(\"%s\"));\n", gettoken(1), getvar("dbobj"), gettokenlist(2, " ", ""));
+    printcode("		\n");
+    nesting = 0;
+    RETURN(1)
+}
+
+int parse_htpl_sql_scope_retrieve(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    if (!nest) RETURN(0)
+    if (!importvar("dbobj", "")) RETURN(croak("Scope  not found in stack"));
+    nesting = 0;
+    RETURN(1)
+}
+
+int parse_htpl_sql_scope(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    eat(&stack, token);
+    if (!strcasecmp(token, "CONNECT")) return parse_htpl_sql_scope_connect(stack, untag);
+    if (!strcasecmp(token, "EXEC")) return parse_htpl_sql_scope_exec(stack, untag);
+    if (!strcasecmp(token, "CURSOR")) return parse_htpl_sql_scope_cursor(stack, untag);
+    if (!strcasecmp(token, "RETRIEVE")) return parse_htpl_sql_scope_retrieve(stack, untag);
+    return 0;
+}
+
 int parse_htpl_sql_msql(stack, untag)
     int untag;
     STR stack; {
@@ -1157,6 +1481,7 @@ int parse_htpl_sql(stack, untag)
         printcode("use HTML::HTPL::Db;\n");
     }
     eat(&stack, token);
+    if (!strcasecmp(token, "BATCH")) return parse_htpl_sql_batch(stack, untag);
     if (!strcasecmp(token, "UPDATE")) return parse_htpl_sql_update(stack, untag);
     if (!strcasecmp(token, "EXEC")) return parse_htpl_sql_exec(stack, untag);
     if (!strcasecmp(token, "MODIFY")) return parse_htpl_sql_modify(stack, untag);
@@ -1173,13 +1498,30 @@ int parse_htpl_sql(stack, untag)
     if (!strcasecmp(token, "QUERY")) return parse_htpl_sql_query(stack, untag);
     if (!strcasecmp(token, "MYSQL")) return parse_htpl_sql_mysql(stack, untag);
     if (!strcasecmp(token, "SEARCH")) return parse_htpl_sql_search(stack, untag);
-    if (!strcasecmp(token, "POSTGRES")) return parse_htpl_sql_postgres(stack, untag);
     if (!strcasecmp(token, "PROJECT")) return parse_htpl_sql_project(stack, untag);
+    if (!strcasecmp(token, "POSTGRES")) return parse_htpl_sql_postgres(stack, untag);
+    if (!strcasecmp(token, "SCOPE")) return parse_htpl_sql_scope(stack, untag);
     if (!strcasecmp(token, "MSQL")) return parse_htpl_sql_msql(stack, untag);
     if (!strcasecmp(token, "CURSOR")) return parse_htpl_sql_cursor(stack, untag);
     return 0;
 }
 
+int parse_htpl_poof(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    printcode("print \"poof\\n\";\n");
+    nesting = 0;
+    RETURN(1)
+}
+
 int parse_htpl_proc___fwd(stack, untag)
     int untag;
     STR stack; {
@@ -1271,6 +1613,20 @@ int parse_htpl_class___fwd(stack, untag)
     printcode("		$self->{$_} = $hash{$_};\n");
     printcode("	}\n");
     printcode("}\n");
+    buff = (STR)mysprintf("#CLSUTILS OTHER\n");
+    nest++;
+    code = parse_htpl(strchr(buff, '#') + 1, 0);
+    nest--;
+    if (!code) {
+        croak("Unification of '%s' failed", buff);
+        free(buff);
+        RETURN(0)
+    }
+    free(buff);
+    printcode("sub __shadow__clone {\n");
+    printcode("	require Clone;\n");
+    printcode("	Clone::clone($self);\n");
+    printcode("}\n");
     nesting = 0;
     RETURN(1)
 }
@@ -1644,8 +2000,6 @@ int parse_htpl_catch(stack, untag)
     static int nesting = 0;
 
     makepersist(stack);
-    if (numtokens > 1) RETURN(croak("CATCH called with %d arguments, maximum needed is 1", numtokens))
-
     if (!currscope) RETURN(croak("Unexpected CATCH"))
     if (currscope->scope != scope_try && currscope->scope != scope_catch) RETURN(croak("Now in scope %s from %d and met CATCH, expecting: try,catch", scope_names[currscope->scope], currscope->nline))
     popscope();
@@ -1655,7 +2009,7 @@ int parse_htpl_catch(stack, untag)
         printcode("$__htpl_default_handler = sub {$_ = shift; \n");
     }
     if (numtokens >= 1)  {
-        printfcode("$__htpl_handler{'%s'} = sub {$_ = shift; \n", gettoken(1));
+        printfcode("$__htpl_handler{'%s'} = sub {$_ = shift; \n", gettokenlist(1, " ", ""));
     }
     nesting = 0;
     RETURN(1)
@@ -2383,31 +2737,119 @@ int parse_htpl_throw(stack, untag)
     static int nesting = 0;
 
     makepersist(stack);
-    buff = (STR)mysprintf("#line %d %s\n", nline, thefilename);
-    nest++;
-    code = parse_htpl(strchr(buff, '#') + 1, 0);
-    nest--;
-    if (!code) {
-        croak("Unification of '%s' failed", buff);
-        free(buff);
-        RETURN(0)
-    }
-    free(buff);
+    printfcode("%#line %d %s\n", nline, thefilename);
     printfcode("die \"%s\";\n", gettokenlist(1, " ", ""));
-    buff = (STR)mysprintf("#line %d %s\n", rline + currbuffer->lines + 1 + 1, thescript);
-    nest++;
-    code = parse_htpl(strchr(buff, '#') + 1, 0);
-    nest--;
-    if (!code) {
-        croak("Unification of '%s' failed", buff);
-        free(buff);
-        RETURN(0)
-    }
-    free(buff);
+    printfcode("%#line %d %s\n", rline + currbuffer->lines + 1 + 1, thescript);
     nesting = 0;
     RETURN(1)
 }
 
+int parse_htpl_pts_set(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    printcode("{\n");
+    if (numtokens < 2) RETURN(croak("PTS SET called with %d arguments, minimum needed is 2", numtokens))
+    if (numtokens > 3) RETURN(croak("PTS SET called with %d arguments, maximum needed is 3", numtokens))
+
+    printfcode("my @t = split(/:/, \"%s\");\n", gettoken(1));
+    printcode("push(@t, $HTML::HTPL::Config::htpl_pts_port);\n");
+    printcode("$HTML::HTPL::pts_obj = new RPC::PlClient(\n");
+    printcode("             peeraddr => $t[0],\n");
+    printcode("             peerport => $t[1],\n");
+    printfcode("             user => \"%s\",\n", gettoken(2));
+    printfcode("             password => \"%s\",\n", gettoken(3));
+    printcode("             application => \"pts\",\n");
+    printcode("             version => 1.0);\n");
+    printcode("}\n");
+    nesting = 0;
+    RETURN(1)
+}
+
+int parse_htpl_pts_create(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    if (numtokens < 2) RETURN(croak("PTS CREATE called with %d arguments, minimum needed is 2", numtokens))
+    printfcode("$%s = $HTML::HTPL::pts_obj->ClientObject(\"%s\", \"new\",\n", gettoken(1), gettoken(2));
+    printfcode("                      qw(%s));\n", gettokenlist(3, " ", ""));
+    nesting = 0;
+    RETURN(1)
+}
+
+int parse_htpl_pts_pool(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    if (numtokens < 3) RETURN(croak("PTS POOL called with %d arguments, minimum needed is 3", numtokens))
+    if (numtokens > 3) RETURN(croak("PTS POOL called with %d arguments, maximum needed is 3", numtokens))
+
+    printfcode("$%s = $HTML::HTPL::pts_obj->ClientObject(\"%s\", \n", gettoken(1), gettoken(2));
+    printfcode(" \"getObject\", \"%s\");\n", gettoken(3));
+    nesting = 0;
+    RETURN(1)
+}
+
+int parse_htpl_pts_call(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    makepersist(stack);
+    if (numtokens < 2) RETURN(croak("PTS CALL called with %d arguments, minimum needed is 2", numtokens))
+    printfcode("$%s = $HTML::HTPL::pts_obj->Call(\"%s\", qw(%s));\n", gettoken(1), gettoken(1), gettokenlist(3, " ", ""));
+    nesting = 0;
+    RETURN(1)
+}
+
+int parse_htpl_pts(stack, untag)
+    int untag;
+    STR stack; {
+
+    TOKEN token;
+    static done = 0;
+    STR buff;
+    int code;
+    static int nesting = 0;
+
+    if (!done) {
+        done = 1;
+        printcode("use RPC::PlClient;\n");
+    }
+    eat(&stack, token);
+    if (!strcasecmp(token, "SET")) return parse_htpl_pts_set(stack, untag);
+    if (!strcasecmp(token, "CREATE")) return parse_htpl_pts_create(stack, untag);
+    if (!strcasecmp(token, "POOL")) return parse_htpl_pts_pool(stack, untag);
+    if (!strcasecmp(token, "CALL")) return parse_htpl_pts_call(stack, untag);
+    return 0;
+}
+
 int parse_htpl_project(stack, untag)
     int untag;
     STR stack; {
@@ -2420,9 +2862,7 @@ int parse_htpl_project(stack, untag)
 
     makepersist(stack);
     if (numtokens < 3) RETURN(croak("PROJECT called with %d arguments, minimum needed is 3", numtokens))
-    if (numtokens > 3) RETURN(croak("PROJECT called with %d arguments, maximum needed is 3", numtokens))
-
-    printfcode("@%s = $%s->project(\"%s\");\n", gettoken(3), gettoken(1), gettoken(2));
+    printfcode("@%s = $%s->project(qw(%s));\n", gettoken(2), gettoken(1), gettokenlist(3, " ", ""));
     nesting = 0;
     RETURN(1)
 }
@@ -2692,8 +3132,7 @@ int parse_htpl_try___fwd(stack, untag)
     makepersist(stack);
     pushscope(scope_try, 0);
     printcode("{\n");
-    printcode("my %__htpl_handler, $__htpl_default_handler; \n");
-    printcode("my $__htpl_try_sub = sub {\n");
+    printcode("my $__htpl__try__sub = sub {\n");
     nesting = 0;
     RETURN(1)
 }
@@ -2711,7 +3150,7 @@ int parse_htpl_try___rev(stack, untag)
     makepersist(stack);
     printcode("};\n");
     printcode("$@ = undef; \n");
-    printcode("eval '&$__htpl_try_sub';\n");
+    printcode("eval '&$__htpl__try__sub;';\n");
     printcode("if ($@) {\n");
     printcode("	foreach (keys %__htpl_handler) {\n");
     printcode("		my $v = $__htpl_handler{$_};\n");
@@ -2763,6 +3202,7 @@ int parse_htpl(stack, untag)
     if (!strcasecmp(token, "LOAD")) return parse_htpl_load(stack, untag);
     if (!strcasecmp(token, "METHOD")) return parse_htpl_method(stack, untag);
     if (!strcasecmp(token, "LDAP")) return parse_htpl_ldap(stack, untag);
+    if (!strcasecmp(token, "REM")) return parse_htpl_rem(stack, untag);
     if (!strcasecmp(token, "INCLUDE")) return parse_htpl_include(stack, untag);
     if (!strcasecmp(token, "DEFAULT")) return parse_htpl_default(stack, untag);
     if (!strcasecmp(token, "CONTINUE")) return parse_htpl_continue(stack, untag);
@@ -2773,6 +3213,7 @@ int parse_htpl(stack, untag)
     if (!strcasecmp(token, "FOR")) return parse_htpl_for(stack, untag);
     if (!strcasecmp(token, "TIME")) return parse_htpl_time(stack, untag);
     if (!strcasecmp(token, "SQL")) return parse_htpl_sql(stack, untag);
+    if (!strcasecmp(token, "POOF")) return parse_htpl_poof(stack, untag);
     if (!strcasecmp(token, "PROC")) return parse_htpl_proc(stack, untag);
     if (!strcasecmp(token, "CLASS")) return parse_htpl_class(stack, untag);
     if (!strcasecmp(token, "PUBLISH")) return parse_htpl_publish(stack, untag);
@@ -2798,6 +3239,7 @@ int parse_htpl(stack, untag)
     if (!strcasecmp(token, "SWITCH")) return parse_htpl_switch(stack, untag);
     if (!strcasecmp(token, "CLSUTILS")) return parse_htpl_clsutils(stack, untag);
     if (!strcasecmp(token, "THROW")) return parse_htpl_throw(stack, untag);
+    if (!strcasecmp(token, "PTS")) return parse_htpl_pts(stack, untag);
     if (!strcasecmp(token, "PROJECT")) return parse_htpl_project(stack, untag);
     if (!strcasecmp(token, "FETCHCOLS")) return parse_htpl_fetchcols(stack, untag);
     if (!strcasecmp(token, "CALL")) return parse_htpl_call(stack, untag);
